interface Options {
    connectionTimeoutMs: number;
    pingIntervalMs: number;
    pingTimeoutMs: number;
}
export declare type OutEnvelope = {
    type: "subscribe";
    id: string;
    message: {
        query: string;
        variables: any;
    };
    extensions?: Record<string, any>;
} | {
    type: "mutate";
    id: string;
    message: {
        query: string;
        variables: any;
    };
    extensions?: Record<string, any>;
} | {
    type: "unsubscribe";
    id: string;
} | {
    type: "echo";
};
export declare type InEnvelope = {
    type: "update";
    id: string;
    message: any;
    metadata?: Record<string, any>;
} | {
    type: "result";
    id: string;
    message: any;
    metadata?: Record<string, any>;
} | {
    type: "error";
    id: string;
    message: string;
    metadata?: Record<string, any>;
} | {
    type: "echo";
};
export declare type ConnectFunction = () => Promise<WebSocket>;
export interface SocketListener {
    onOpen(): void;
    onMessage(message: any): void;
    onClose(reason: string): void;
}
export declare class PingingWebSocket {
    static readonly defaultOptions: Options;
    static readonly reasonPingTimeout: string;
    static readonly reasonConnectionTimeout: string;
    static readonly reasonCloseCalled: string;
    private readonly connectFunction;
    private readonly listener;
    private readonly options;
    private socket?;
    private hasShutdown;
    private connectTimeout?;
    private sendPingTimeout?;
    private receivePingTimeout?;
    constructor(connectFunction: ConnectFunction, listener: SocketListener, options?: Partial<Options>);
    send(message: OutEnvelope): void;
    close(): void;
    readonly readyState: number;
    connect(): Promise<void>;
    private sendPing;
    private shutdown;
}
export default PingingWebSocket;
